# nntplib wrapper

import timeoutsocket
import pynenntp
from boxtypes.net_exceptions import *
import pynei18n
import time
import string
import cStringIO

# Stupid python 1.5.x pynenntp...
error_nntp = (pynenntp.error_reply, pynenntp.error_temp, pynenntp.error_perm, pynenntp.error_proto, pynenntp.error_data)

def gmtime_2_nntptime(gmtime):
	"""
	Convert gmtime to something nntp likes. Returns tuple:
	("yymmdd", "hhmmss").
	"""
	yymmdd = time.strftime("%y%m%d", gmtime)
	hhmmss = time.strftime("%H%M%S", gmtime)

	return (yymmdd, hhmmss)

class nntp_io:
	def __init__(self, nntpbox):
		# Remember for config
		self.nntpbox = nntpbox
	
	def renew_connection(self):
		try:
			if self.nntpbox.username == "" and self.nntpbox.password == "":
				self.connection = pynenntp.NNTP(self.nntpbox.server, self.nntpbox.port)
			elif self.nntpbox.password == "":
				self.connection = pynenntp.NNTP(self.nntpbox.server, self.nntpbox.port, self.nntpbox.username)
			else:
				self.connection = pynenntp.NNTP(self.nntpbox.server, self.nntpbox.port, self.nntpbox.username, self.nntpbox.password)
		except (timeout_exception, pynenntp.error_perm, pynenntp.error_temp, EOFError), e:
			raise connect_exception(e)
	
	def check_renew_connection(self):
		"""
		Renew connection if it died.
		"""
		try:
			# this is probably a silly way of doing it
			self.connection.stat("<hi@server>")
		except (pynenntp.error_temp, pynenntp.error_proto):
			# no such article. healthy connection ;-)
			pass
		except (timeout_exception, pynenntp.error_perm, AttributeError):
			# dead. renew it.
			self.renew_connection()
			
	def get_server_time(self):
		"""
		Get time, date strings from server or just
		use local time if the server doesn't support.
		"""
		try:
			new_time = self.connection.date()
		except (timeout_exception, pynenntp.NNTPError), e:
			try:
				# python 1.5 and 2.x nntp exceptions are
				# totally incompatible...
				e = e.args[0]
			except AttributeError:
				pass
			if string.split(e)[0] == "503":
				# 503 connection timeout. freak out
				return None
			else:
				# not supported. just do it the lame way
				new_time = (None,) + gmtime_2_nntptime(time.gmtime(time.time()))
		return new_time

	def get_header(self, id):
		"""
		Return headers of message 'id' from NNTP server.
		"""
		try:
			lines = self.connection.head(id)[3]
		except pynenntp.error_temp:
			# article not on server
			return None
		except EOFError, e:
			# god hates us (and the server is a dick)
			raise recv_exception(e)
		except pynenntp.error_perm:
			raise "NNTP_cockup_error"
		head = string.join(lines, "\n")
		# i've seen nulls in mesages. bad.
		head = string.replace(head, "\000", "")
		return head

	def get_body(self, id):
		"""
		Return body of message 'id' from NNTP server.
		"""
		try:
			lines = self.connection.body(id)[3]
		except pynenntp.error_temp, e:
			# article not on server
			raise recv_exception(e)
		except EOFError, e:
			# god hates us (and the server is a dick)
			raise recv_exception(e)
		body = string.join(lines, "\n")
		# evil nulls in string. crashes GTK+
		body = string.replace(body, "\000", "")
		return body

	def delete_msg(self, id):
		pass

	def post_message(self, msg):
		msg_body = cStringIO.StringIO(msg.body)
		try:
			self.connection.post(msg_body)
		except error_nntp, e:
			raise send_exception(e)

	def close(self):
		self.connection.quit()

	# Just do the damn thing...
	def group(self, groupname):
		return self.connection.group(groupname)
	def xhdr(self, header, argument, callback=None):
		return self.connection.xhdr(header, argument, callback)
	def list(self, callback=None):
		"""
		Callback should take an int argument. This will be how many
		groups done so far.
		"""
		return self.connection.list(callback)
	def newgroups(self, yymmdd, hhmmss, callback=None):
		return self.connection.newgroups(yymmdd, hhmmss, callback)

def get_connection(nntpbox, no_connection=0):
	n = nntp_io(nntpbox)
	if not no_connection:
		n.renew_connection()
	return n

